<?php

require_once('../init.php');

function fetch($mask)
{
  global $db;

  $fields = array("to_char(sys_edit,'DD.MM.YYYY HH24:mm') AS sys_edit", 'sys_user');
  foreach ($mask['edit'] as $field => $info) {
    if ($info['sql'] === false) continue;
    if ($info['type'] == 'date')
      $fields[] = sprintf("to_char(%s,'DD.MM.YYYY') AS %s",
			  empty($info['sql']) ? $field : $info['sql'],
			  $field);
    elseif ($info['type'] != 'passwd')
      $fields[] = empty($info['sql']) ? $field : $info['sql'] . ' AS ' . $field;
  }

  $sql = sprintf('SELECT id,%s FROM %s WHERE id = %d',
		 implode(',', $fields),
		 $mask['table'], $_POST['id']);

  $sth = $db->query($sql);
  if ($sth === false) return false;

  $row = $sth->fetch();

  foreach ($mask['edit'] as $field => $info)
    if ($info['type'] == 'boolean')
      $row[$field] = $row[$field]?true:false;
    elseif ($info['type'] == 'passwd')
      $row[$field] = '';
    elseif (array_key_exists('format', $info))
      $row[$field] = sprintf($info['format'], $row[$field]);

  return $row;
}

function details($mask)
{
  if (empty($_POST['id']))
    return array('error' => 'Missing ID');

  $fields = array();
  foreach ($mask['details']['list'] as $field => $info) {
    if ($info['type'] == 'date')
      $fields[] = sprintf("to_char(%s,'DD.MM.YYYY') AS %s",
			  empty($info['sql']) ? $field : $info['sql'],
			  $field);
    elseif (!array_key_exists('fetch',$info))
      $fields[] = empty($info['sql']) ? $field : $info['sql'] . ' AS ' . $field;
  }

  if (count($fields)) {
    $sql = sprintf('SELECT id,%s FROM %s WHERE id = %d',
		   implode(',', $fields),
		   $mask['table'], $_POST['id']);

    $sth = $db->query($sql);

    if (!$sth)
      return array('error' => db_error(),
		   'sql' => $sql);

    $row = $sth->fetch();
  } else {
    $row = array();
  }

  foreach ($mask['details']['list'] as $field => $info)
    if (array_key_exists('format', $info))
      $row[$field] = sprintf($info['format'], $row[$field]);
    elseif (array_key_exists('fetch', $info))
      $row[$field] = $info['fetch']();

  return $row;
}

function format_date($value)
{
  $d = explode('.', $value);
  if (count($d) == 3)
    $value = sprintf('%d-%d-%d', $d[2], $d[1], $d[0]);
  return $value;
}

function format_decimal($value)
{
  $value = str_replace(',','.',$value);
  return sprintf("%.2f", $value);
}

$upload_error = array(
		      'There is no error.',
		      'The uploaded file exceeds the upload_max_filesize directive.',
		      'The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the form.',
		      'The uploaded file was only partially uploaded.',
		      'No file was uploaded.',
		      'Missing a temporary folder.',
		      'Failed to write file to disk.',
		      'A PHP extension stopped the file upload.'
		      );

function store_file($field, $info)
{
  global $upload_error;

  if (empty($_FILES[$field]['name']))
    return array('error' => 'Kein Dateiname angegeben',
		 'errormsg' => 'Kein Dateiname angegeben');

  if ($_FILES[$field]['error'] > 0)
    return array('error' => "Fehler beim Upload einer Datei:\n" . $upload_error[$_FILES[$field]['error']],
		 'errormsg' => 'Fehler beim Upload einer Datei');

  if (empty($_FILES[$field]['tmp_name']))
    return array('error' => 'Keine temporäre Datei erzeugt',
		 'errormsg' => 'Keine temporäre Datei erzeugt');

  if (empty($info['path']))
    return array('error' => 'Kein Pfadname konfiguriert',
		 'errormsg' => 'Kein Pfadname konfiguriert');

  $parts = pathinfo($_FILES[$field]['name']);

  $base = $_SESSION['sys']['basedir'] . 'archive';
  if (!is_dir($base))
    return array('error' => 'Archivpfad nicht vorhanden',
		 'errormsg' => 'Archivpfad nicht vorhanden');

  $base .= '/' . $info['path'];
  if (!is_dir($base))
    return array('error' => 'Dokumentenpfad nicht vorhanden',
		 'errormsg' => 'Dokumentenpfad nicht vorhanden');
  $base .= '/';

  $path = date('Y');
  if (!is_dir($base.$path))
    if (mkdir($base.$path) === false)
      return array('error' => 'Kann Verzeichnis nicht anlegen: ' . $path,
		   'errormsg' => 'Kann Verzeichnis nicht anlegen');

  $path .= '/' . date('m');
  if (!is_dir($base.$path))
    if (mkdir($base.$path) === false)
      return array('error' => 'Kann Verzeichnis nicht anlegen: ' . $path,
		   'errormsg' => 'Kann Verzeichnis nicht anlegen');

  $path .= '/' . $parts['filename'];

  $max = 100;
  for ($i=0; $i < $max; $i++) {
    $fname = $path;
    if ($i > 0) $fname .= '('.$i.')';
    if (!empty($parts['extension']))
      $fname .= '.' . $parts['extension'];

    if (!is_file($base.$fname)) {
      if (move_uploaded_file($_FILES[$field]['tmp_name'], $base.$fname) === false)
	return array('error' => 'Kann Datei nicht speichern ',
		     'errormsg' => 'Kann Datei nicht speichern');

      chmod($base.$fname, 0640);
      return $fname;
    }
  }

  return array('error' => 'Kann Datei nicht anlegen',
	       'errormsg' => 'Kann Datei nicht anlegen');
}

function save($mask)
{
  global $db;

  if (array_key_exists('save', $mask))
    return $mask['save']($mask);

  if (empty($_POST['id']))
    return array('error' => 'Missing ID');

  $update = array(sprintf("sys_user = %s", $db->quote($_SESSION['sys']['login'])),
		  "sys_edit = now()");

  foreach ($mask['edit'] as $field => $info) {
    if ($info['type'] == 'html') continue;
    if ($info['type'] == 'file') continue;

    if ($info['required'] === true && !strlen($_POST[$field]))
      return array('error' => sprintf('Pflichtfeld %s nicht ausgefüllt', $info['name']),
		   'errormsg' => 'Pflichtfelder nicht ausgefüllt');

    if ($info['type'] == 'boolean') {
      $update[] = sprintf("%s=%d", $field, $_POST[$field] == 'on'?1:0);
    } elseif ($info['type'] == 'hidden') {
      if (empty($_POST[$field]) && $info['null'] === true)
        $update[] = sprintf("%s=NULL", $field);
      else {
	if ($info['subtype'] == 'string')
	  $update[] = sprintf("%s=%s", $field, $db->quote($_POST[$field]));
	else
	  $update[] = sprintf("%s=%d", $field, $_POST[$field]);
      }
    } elseif ($info['type'] == 'number' || ($info['type'] == 'select' && $info['options_string'] !== true)) {
      if (empty($_POST[$field]) && $info['null'] === true)
        $update[] = sprintf("%s=NULL", $field);
      else
	$update[] = sprintf("%s=%d", $field, $_POST[$field]);
    } elseif ($info['type'] == 'decimal') {
      if (empty($_POST[$field]) && $info['null'] === true)
        $update[] = sprintf("%s=NULL", $field);
      else
	$update[] = sprintf("%s=%s", $field, format_decimal($_POST[$field]));
    } elseif ($info['type'] == 'date') {
      if (empty($_POST[$field]) && $info['null'] === true)
        $update[] = sprintf("%s=NULL", $field);
      else
	$update[] = sprintf("%s='%s'", $field, format_date($_POST[$field]));
    } elseif ($info['type'] == 'passwd') {
      if (!empty($_POST[$field]))
	$update[] = sprintf("%s=%s", $field,
			    $db->quote(passwd(empty($_POST['login'])?$_SESSION['sys']['login']:$_POST['login'],
						    $_POST[$field])));
    } else {
      if (empty($_POST[$field]) && $info['null'] === true)
        $update[] = sprintf("%s=NULL", $field);
      else
        $update[] = sprintf("%s=%s", $field, $db->quote($_POST[$field]));
    }
  }

  $sql = sprintf('UPDATE %s SET %s WHERE id = %d',
		 empty($mask['edit_table']) ? $mask['table'] : $mask['edit_table'],
		 implode(', ', $update),
		 intval($_POST['id']));

  $sth = $db->query($sql);

  if ($sth === false) {
    error_log($sql . ': ' . db_error());
    return array('error' => db_error(),
                 'sql' => $sql);
  }

  return array('status' => true);
}

function insert($mask)
{
  global $db;

  if (array_key_exists('insert', $mask))
    return $mask['insert']($mask);

  $fields = array('sys_user','sys_edit');
  $values = array($db->quote($_SESSION['sys']['login']), 'now()');

  foreach ($mask['edit'] as $field => $info) {
    if ($info['type'] == 'html') continue;

    if ($info['required'] === true && !strlen($_POST[$field]))
      return array('error' => sprintf('Pflichtfeld %s nicht ausgefüllt', $info['name']),
		   'errormsg' => 'Pflichtfelder nicht ausgefüllt');

    if ($info['type'] == 'boolean') {
      $fields[] = $field;
      $values[] = $_POST[$field] == 'on'?1:0;
    } elseif ($info['type'] == 'hidden') {
      $fields[] = $field;
      if (empty($_POST[$field]) && $info['null'] === true)
	$values[] = 'NULL';
      else {
	if ($info['subtype'] == 'string')
	  $values[] = $db->quote($_POST[$field]);
	else
	  $values[] = intval($_POST[$field]);
      }
    } elseif ($info['type'] == 'number' || ($info['type'] == 'select' && $info['options_string'] !== true)) {
      $fields[] = $field;
      if (empty($_POST[$field]) && $info['null'] === true)
	$values[] = 'NULL';
      else
	$values[] = intval($_POST[$field]);
    } elseif ($info['type'] == 'decimal') {
      $fields[] = $field;
      if (empty($_POST[$field]) && $info['null'] === true)
	$values[] = 'NULL';
      else
	$values[] = format_decimal($_POST[$field]);
    } elseif ($info['type'] == 'date') {
      $fields[] = $field;
      if (empty($_POST[$field]) && $info['null'] === true)
	$values[] = 'NULL';
      else
	$values[] = "'" . format_date($_POST[$field]) . "'";
    } elseif ($info['type'] == 'passwd') {
      if (!empty($_POST[$field])) {
	$fields[] = $field;
	$values[] = $db->quote(passwd(empty($_POST['login'])?$_SESSION['sys']['login']:$_POST['login'],
				      $_POST[$field]));
      }
    } elseif ($info['type'] == 'file') {
      $fields[] = $field;
      if (empty($_FILES[$field]) && $info['null'] === true)
        $values[] = 'NULL';
      elseif (empty($_FILES[$field]))
        $values[] = "''";
      else {
	$fname = store_file($field, $info);
	if (is_array($fname))
	  return $fname;
        $values[] = $db->quote($fname);
      }

    } else {
      $fields[] = $field;
      if (empty($_POST[$field]) && $info['null'] === true)
	$values[] = 'NULL';
      else
	$values[] = $db->quote(utf8_encode($_POST[$field]));
    }
  }

  $sql = sprintf('INSERT INTO %s (%s) VALUES (%s)',
                 empty($mask['edit_table']) ? $mask['table'] : $mask['edit_table'],
		 implode(',', $fields),
		 implode(',', $values));

  $sth = $db->query($sql);

  if ($sth === false) {
    error_log($sql . ': ' . db_error());
    return array('error' => db_error(),
                 'sql' => $sql);
  }

  $id = db_lastid(empty($mask['edit_table']) ? $mask['table'] : $mask['edit_table']);

  if (array_key_exists('postinsert',$mask))
    $mask['postinsert']($id);

  return array('status' => true, 'id' => $id);
}

function delete_or_copy_row($table,$id,$mask)
{
  global $db;

  if (array_key_exists('predelete',$mask))
    $mask['predelete']($id);

  if (DELETE_COPY === true) {
    $sql = sprintf("INSERT INTO %s_deleted SELECT * FROM %s WHERE id = %d",
		   $table, $table, $id);

    $sth = $db->query($sql);

    if ($sth === false) {
      error_log($sql . ': ' . db_error());
      return array('error' => db_error(),
		   'sql' => $sql);
    }

    $sql = sprintf("UPDATE %s_deleted SET sys_user='%s',sys_edit=now() WHERE id = %d",
		   $table, $_SESSION['sys']['login'], $id);

    $sth = $db->query($sql);

    if ($sth === false) {
      error_log($sql . ': ' . db_error());
      return array('error' => db_error(),
		   'sql' => $sql);
    }
  } else {
    $fields = array();
    foreach ($mask['edit'] as $field => $info)
      if ($info['type'] === 'file')
	$fields[] = array('name' => $field, 'path' => $info['path']);

    if (count($fields)) {
      $f = array();
      foreach ($fields as $i)
	$f[] = $i['name'];

      $sql = sprintf("SELECT %s FROM %s WHERE id = %d", implode(',',$f), $table, $id);
      $sth = $db->query($sql);

      if ($sth === false) {
	error_log($sql . ': ' . db_error());
	return array('error' => db_error(),
		     'sql' => $sql);
      }

      $row = $sth->fetch();
      foreach ($fields as $f) {
	$base = $_SESSION['sys']['basedir'] . 'archive/' . $f['path'];
	$fname = $base . '/' . $row[$f['name']];
	if (is_file($fname))
	  unlink($fname);
      }
    }
  }

  $sql = sprintf("DELETE FROM %s WHERE id = %d", $table, $_POST['id']);

  $sth = $db->query($sql);

  if ($sth === false) {
    error_log($sql . ': ' . db_error());
    return array('error' => db_error(),
                 'sql' => $sql);
  }

  return array('status' => true);
}

function delete_or_copy($mask)
{
  if (empty($_POST['id']))
    return array('error' => 'Missing ID');

  if (array_key_exists('delete', $mask))
    return $mask['delete']($mask);

  if (!empty($mask['edit_table']))
    return array('error' => 'Cannot handle deletion for secondary table');

  return delete_or_copy_row($mask['table'],$_POST['id'],$mask);
}

function set_variable($name,$mask)
{
  if (!array_key_exists('variables',$mask))
    return array('error' => 'Unknown variable ' . htmlspecialchars($_POST['name']));

  if (!array_key_exists($_POST['name'],$mask['variables']))
    return array('error' => 'Unknown variable ' . htmlspecialchars($_POST['name']));

  $_SESSION[$name . '.' . $_POST['name']] = $_POST['value'];

  if (array_key_exists('postcall',$mask['variables'][$_POST['name']]))
    $mask['variables'][$_POST['name']]['postcall']();

  return array('status' => true);
}

function get_infos($mask)
{
  if (!array_key_exists('info',$mask))
    return array('error' => 'Unknown callback ' . htmlspecialchars($_POST['name']));

  if (!array_key_exists($_POST['name'],$mask['info']))
    return array('error' => 'Unknown callback ' . htmlspecialchars($_POST['name']));

  if (!array_key_exists('sql',$mask['info'][$_POST['name']]))
    return array('error' => 'Unknown callback ' . htmlspecialchars($_POST['name']));

  $sql = $mask['info'][$_POST['name']]['sql'];

  while (preg_match('/\{([^\}]*)\}/', $sql, $matches))
    $sql = str_replace('{'.$matches[1].'}', $_POST[$matches[1]], $sql);

  return array('info' => query_db($sql),
	       'parameter' => $_POST);
}

function custom_function($mask)
{
  if (!array_key_exists('callbacks',$mask))
    return array('error' => 'Unknown callback ' . htmlspecialchars($_POST['callback']));

  if (!array_key_exists($_POST['callback'],$mask['callbacks']))
    return array('error' => 'Unknown callback ' . htmlspecialchars($_POST['callback']));

  return $mask['callbacks'][$_POST['callback']]();
}

function download_file($table,$column,$path,$id)
{
  global $db;
  global $mask;

  $sql = sprintf("SELECT %s FROM %s WHERE id = %d", $column, $table, $id);
  $sth = $db->query($sql);
  if ($sth === false) return "Database error, Query " . $sql;

  $row = $sth->fetch();

  $fname = $_SESSION['sys']['basedir'] . 'archive/' . $path . '/' . $row[$column];

  if (($f = fopen($fname,'r')) === false)
    return sprintf("Cannot open file for id %d", $id);

  $basename = basename($row[$column]);
  $filesize = filesize($fname);
  $content_type = mime_content_type($fname);

  header(sprintf('Content-disposition: inline; filename="%s"', $basename));
  header(sprintf('Content-Length: %d', $filesize));
  header(sprintf('Content-Type: %s', $content_type));
  header("Cache-Control: ");
  header("Pragma: ");
  usleep(5000);
  fpassthru($f);
  fclose($f);
}

function process_file($mask)
{
  if (!array_key_exists('files',$mask))
    return array('error' => 'Unknown function ' . htmlspecialchars($_GET['name']));

  if (!array_key_exists($_GET['name'],$mask['files']))
    return array('error' => 'Unknown callback ' . htmlspecialchars($_GET['name']));

  $mask['files'][$_GET['name']]();
  exit;
}

if (empty($_REQUEST['func']))
  exit;

if (empty($_REQUEST['source']))
  exit;

db_connect();
if (load_mask($_REQUEST['source']) === false) exit;

$data = array('error' => 'Unknown function');

if ($_GET['func'] == 'file' && !empty($_GET['name']))
  $data = process_file($mask);

if ($_POST['func'] == 'fetch') {
  $data = fetch($mask);
} elseif ($_POST['func'] == 'details') {
  $data = details($mask);
} elseif ($_POST['func'] == 'save') {
  $data = save($mask);
} elseif ($_POST['func'] == 'insert') {
  $data = insert($mask);
} elseif ($_POST['func'] == 'delete') {
  $data = delete_or_copy($mask);
} elseif ($_POST['func'] == 'setvar') {
  $data = set_variable($_POST['source'],$mask);
} elseif ($_POST['func'] == 'info') {
  $data = get_infos($mask);
} elseif ($_POST['func'] == 'function') {
  $data = custom_function($mask);
}

format_ajax($data);

?>
